import { IListable } from '../interfaces/listable-interface';

export abstract class DefaultListable<T> implements IListable<T> {

    private _search: boolean;
    private _elements: Array<T>;
    private _elementsAux: Array<T>;
    private _visibleElements: Array<T>;
    private _visibleRange: [number, number] = [0, 20];

    constructor(
    ) {
        this._search = false;
        this._elements = [];
        this._elementsAux = [];
        this._visibleElements = [];
    }

    /**
     *
     */
    set elements(_elements: Array<T>) {
        this._elements = _elements;
        this.visibleElements = _elements.slice(this._visibleRange[0], this._visibleRange[1]);
    }

    /**
     *
     */
    get elements() {
        return this._elements;
    }

    /**
     *
     */
    set visibleElements(_visibleElements: Array<T>) {
        this._visibleElements = _visibleElements;
    }

    /**
     *
     */
    get visibleElements() {
        return this._visibleElements;
    }

    /**
     *
     */
    add(T: any): void {
        this._elements.push(T);
    }

    /**
     *
     */
    remove(T: any): void {
        let index = this.elements.indexOf(T);

        this._elements.splice(index, 1);
        this._visibleElements.splice(index, 1);
    }

    /**
     *
     */
    get(T: any): T {
        let index = this.elements.indexOf(T);
        return this._elements[index];
    }

    /**
     *
     */
    list(): Array<T> {
        return this._elements;
    }

    /**
     *
     */
    search(event: any): void {
        let value: string = event.target.value;

        if (!value) {
            return;
        }

        if (value.length < 3) {
            return;
        }

        this.visibleElements = this.elements.filter(item => {
            return (item as any).name.toLowerCase().indexOf(value.toLowerCase()) > -1;
        });
    }

    /**
     *
     */
    toggleSearch(): void {
        this._search = !this._search;

        if (this._search) {
            this._elementsAux = this.visibleElements;
        } else {
            this.visibleElements = this._elementsAux;
        }
    }

    /**
     *
     */
    isSearchMode(): boolean {
        return this._search;
    }

    /**
     *
     */
    seekRange(): void {
        this._visibleRange[0] = this._visibleRange[1];
        this._visibleRange[1] = this._visibleRange[1] + 20;

        this._visibleElements = this._visibleElements.concat(this._elements.slice(this._visibleRange[0], this._visibleRange[1]));
        console.info("seeking");
    }

    /**
     *
     */
    isToSeek(): boolean {
        return this._visibleRange[1] >= this.elements.length;
    }

    /**
     *
     */
    seek(event: any) {
        this.seekRange();

        if (this.isToSeek()) {
            event.enable(false);
        }

        event.complete();
    }
}
